@@ -27,7 +27,6 @@
 #include <QtGui/qaccessible.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qwindow.h>
-#include <qpa/qplatforminputcontextfactory_p.h>
 #include <QtCore/private/qcomvariant_p.h>
 
 #if !defined(Q_CC_BOR) && !defined (Q_CC_GNU)
@@ -173,6 +172,18 @@ void QWindowsUiaMainProvider::notifyNameChange(QAccessibleEvent *event)
     }
 }
 
+void QWindowsUiaMainProvider::notifyRoleChange(QAccessibleEvent *event)
+{
+    if (QAccessibleInterface *accessible = event->accessibleInterface()) {
+        if (auto provider = providerForAccessible(accessible)) {
+            QComVariant oldVal;
+            QComVariant newVal{ roleToControlTypeId(accessible->role()) };
+            UiaRaiseAutomationPropertyChangedEvent(provider.Get(), UIA_ControlTypePropertyId,
+                                                   oldVal.get(), newVal.get());
+        }
+    }
+}
+
 void QWindowsUiaMainProvider::notifySelectionChange(QAccessibleEvent *event)
 {
     if (QAccessibleInterface *accessible = event->accessibleInterface()) {
@@ -521,10 +532,6 @@ HRESULT QWindowsUiaMainProvider::GetPropertyValue(PROPERTYID idProp, VARIANT *pR
         // Accelerator key.
         *pRetVal = QComVariant{ accessible->text(QAccessible::Accelerator) }.release();
         break;
-    case UIA_AriaRolePropertyId:
-        if (accessible->role() == QAccessible::Heading)
-            *pRetVal = QComVariant{ QStringLiteral("heading") }.release();
-        break;
     case UIA_AriaPropertiesPropertyId:
         setAriaProperties(accessible, pRetVal);
         break;
@@ -574,36 +581,19 @@ HRESULT QWindowsUiaMainProvider::GetPropertyValue(PROPERTYID idProp, VARIANT *pR
             *pRetVal = QComVariant{ UIA_WindowControlTypeId }.release();
         } else {
             // Control type converted from role.
-            auto controlType = roleToControlTypeId(accessible->role());
-
-            // The native OSK should be disabled if the Qt OSK is in use,
-            // or if disabled via application attribute.
-            static bool imModuleEmpty = QPlatformInputContextFactory::requested().isEmpty();
-            bool nativeVKDisabled = QCoreApplication::testAttribute(Qt::AA_DisableNativeVirtualKeyboard);
-
-            // If we want to disable the native OSK auto-showing
-            // we have to report text fields as non-editable.
-            if (controlType == UIA_EditControlTypeId && (!imModuleEmpty || nativeVKDisabled))
-                controlType = UIA_TextControlTypeId;
-
-            *pRetVal = QComVariant{ controlType }.release();
+            *pRetVal = QComVariant{ roleToControlTypeId(accessible->role()) }.release();
         }
         break;
     case UIA_HelpTextPropertyId:
         *pRetVal = QComVariant{ accessible->text(QAccessible::Help) }.release();
         break;
     case UIA_HasKeyboardFocusPropertyId:
-        // If the top-level window has no focused child, report the top-level
-        // widget (window). If it already has a focused widget, it will be
-        // reported automatically.
         if (topLevelWindow) {
-            QAccessibleInterface *focusacc = accessible->focusChild();
-            if (!focusacc) {
-                *pRetVal = QComVariant{ accessible->state().active ? true : false }.release();
-                break;
-            }
+            // Windows set the active state to true when they are focused
+            *pRetVal = QComVariant{ accessible->state().active ? true : false }.release();
+        } else {
+            *pRetVal = QComVariant{ accessible->state().focused ? true : false }.release();
         }
-        *pRetVal = QComVariant{ accessible->state().focused ? true : false }.release();
         break;
     case UIA_IsKeyboardFocusablePropertyId:
         if (topLevelWindow) {
